@@ -18,9 +18,9 @@
 #
 from checkbox.attribute import Attribute
 from checkbox.variables import (ConstantVariable, BoolVariable, StringVariable,
-    PathVariable, UnicodeVariable, IntVariable, FloatVariable, ListVariable,
-    TupleVariable, AnyVariable, DictVariable, MapVariable, VariableFactory,
-    Variable, get_variable)
+    PathVariable, UnicodeVariable, IntVariable, FloatVariable, TimeVariable,
+    ListVariable, TupleVariable, AnyVariable, DictVariable, MapVariable,
+    VariableFactory, Variable, get_variable)
 
 
 class Property(object):
@@ -47,7 +47,7 @@ class Property(object):
     def _detect_name(self, used_cls):
         self_id = id(self)
         for cls in used_cls.__mro__:
-            for attr, prop in cls.__dict__.items():
+            for attr, prop in cls.__dict__.iteritems():
                 if id(prop) == self_id:
                     return attr
         raise RuntimeError("Property used in an unknown class")
@@ -141,6 +141,11 @@ class Float(PropertyType):
     variable_class = FloatVariable
 
 
+class Time(PropertyType):
+
+    variable_class = TimeVariable
+
+
 class List(PropertyFactory):
 
     variable_class = ListVariable
@@ -183,7 +188,7 @@ class Map(PropertyType):
     variable_class = MapVariable
 
     def __init__(self, schema={}, **kwargs):
-        for key, type in schema.items():
+        for key, type in schema.iteritems():
             schema[key] = VariableFactory(type._variable_class,
                 **type._variable_kwargs)
 
