@@ -23,7 +23,6 @@ import termios
 
 from gettext import gettext as _
 
-from checkbox.job import Job
 from checkbox.user_interface import (UserInterface, ANSWER_TO_STATUS,
     ALL_ANSWERS, YES_ANSWER, NO_ANSWER, SKIP_ANSWER)
 
@@ -54,6 +53,7 @@ class CLIDialog(object):
 
     def get(self, label=None, limit=1, separator=termios.CEOT):
         if label is not None:
+            self.put_newline()
             self.put(label)
 
         fileno = sys.stdin.fileno()
@@ -94,7 +94,6 @@ class CLIDialog(object):
         finally:
             termios.tcsetattr(fileno, termios.TCSANOW, saved_attributes)
 
-        self.put_newline()
         return "".join(input)
 
     def show(self):
@@ -110,29 +109,36 @@ class CLIChoiceDialog(CLIDialog):
         self.keys = []
         self.options = []
 
+    def get(self, *args, **kwargs):
+        response = super(CLIChoiceDialog, self).get(*args, **kwargs)
+        try:
+            return self.keys.index(response[0])
+        except ValueError:
+            return -1
+
     def run(self, label=None, defaults=[]):
         if not self.visible:
             self.show()
 
-        self.put_newline()
         try:
             # Only one option
-            if len (self.keys) <= 1:
+            if len(self.keys) <= 1:
                 self.get(_("Press any key to continue..."))
                 return 0
             # Multiple choices
             while True:
-                if label is not None:
-                    self.put_line(label)
+                self.put_newline()
                 for key, option in zip(self.keys, self.options):
                     default = "*" if option in defaults else " "
                     self.put_line("%s %s: %s" % (default, key, option))
 
                 response = self.get(_("Please choose (%s): ") % ("/".join(self.keys)))
-                try:
-                    return self.keys.index(response[0]) + 1
-                except ValueError:
-                    pass
+                if response >= 0:
+                    return response
+
+                if label is not None:
+                    self.put_line(label)
+
         except KeyboardInterrupt:
             self.put_newline()
             raise
@@ -155,8 +161,7 @@ class CLITextDialog(CLIDialog):
 
         self.put_newline()
         try:
-            response = self.get(label, self.limit, self.separator)
-            return response
+            return self.get(label, self.limit, self.separator)
         except KeyboardInterrupt:
             self.put_newline()
             raise
@@ -189,6 +194,31 @@ class CLIProgressDialog(CLIDialog):
 
 class CLIInterface(UserInterface):
 
+    def _toggle_results(self, key, options, results):
+        if isinstance(results, dict):
+            if key in results:
+                del results[key]
+
+            elif key in options:
+                if isinstance(options[key], dict):
+                    results[key] = {}
+                elif isinstance(options[key], (list, tuple,)):
+                    results[key] = []
+                else:
+                    raise Exception, "Unknown result type: %s" % type(results)
+
+                for k in options[key]:
+                    self._toggle_results(k, options[key], results[key])
+
+        elif isinstance(results, (list, tuple,)):
+            if key in results:
+                results.remove(key)
+            elif key in options:
+                results.append(key)
+
+        else:
+            raise Exception, "Unknown result type: %s" % type(results)
+
     def show_progress_start(self, text):
         self.progress = CLIProgressDialog(text)
         self.progress.show()
@@ -208,45 +238,82 @@ class CLIInterface(UserInterface):
     def show_check(self, text, options=[], default=[]):
         dialog = CLIChoiceDialog(text)
         for option in options:
-            dialog.add_option(option.capitalize())
+            dialog.add_option(option)
 
         dialog.add_option("Space when finished", " ")
 
-        results = dict((d, d in default) for d in options)
+        results = default
         while True:
-            defaults = [k for k, v in results.items() if v]
-            response = dialog.run(defaults=defaults)
-            if response > len(options):
+            response = dialog.run(defaults=results)
+            if response >= len(options):
                 break
 
-            result = options[response - 1]
-            results[result] = not results[result]
+            result = options[response]
+            self._toggle_results(result, options, results)
 
-        return [k for k, v in results.items() if v]
+        return results
 
     def show_radio(self, text, options=[], default=None):
         dialog = CLIChoiceDialog(text)
         for option in options:
-            dialog.add_option(option.capitalize())
+            dialog.add_option(option)
 
         # Show options dialog
         response = dialog.run()
-        return options[response - 1]
+        return options[response]
 
-    def _run_test(self, test):
-        message = _("Running test %s...") % test["name"]
-        job = Job(test["command"], test.get("environ"),
-            test.get("timeout"), test.get("user"))
-        (status, data, duration) = self.show_progress(message, job.execute)
-        return data
+    def show_tree(self, text, options={}, default={}):
+        keys = sorted(options.keys())
+        values = [options[k] for k in keys]
 
-    def show_test(self, test):
+        dialog = CLIChoiceDialog(text)
+        for option in keys:
+            dialog.add_option(option)
+
+        dialog.add_option("Plus to expand options", "+")
+        dialog.add_option("Space when finished", " ")
+
+        do_expand = False
+        results = default
+        while True:
+            response = dialog.run(defaults=results)
+            if response > len(options):
+                break
+
+            elif response == len(options):
+                response = dialog.get()
+                do_expand = True
+
+            else:
+                do_expand = False
+
+            # Invalid response
+            if response < 0:
+                continue
+
+            # Toggle results
+            result = keys[response]
+            if not do_expand:
+                self._toggle_results(result, options, results)
+                continue
+
+            # Expand tree
+            dialog.visible = False
+            if options[result]:
+                branch_results =  results.get(result, {})
+                self.show_tree(result, options[result], branch_results)
+                if branch_results and result not in results:
+                    results[result] = branch_results
+
+        return results
+
+    def show_test(self, test, runner):
         options = list([ANSWER_TO_OPTION[a] for a in ALL_ANSWERS])
         if "command" in test:
             options.append(_("test"))
 
         if re.search(r"\$output", test["description"]):
-            output = self._run_test(test)
+            output = runner(test)[1]
         else:
             output = ""
 
@@ -257,15 +324,15 @@ class CLIInterface(UserInterface):
             dialog = CLIChoiceDialog(description)
 
             for option in options:
-                dialog.add_option(option.capitalize(), option[0])
+                dialog.add_option(option, option[0])
 
             # Get option from dialog
             response = dialog.run()
-            option = options[response - 1]
-            if response <= len(ALL_ANSWERS):
+            option = options[response]
+            if response < len(ALL_ANSWERS):
                 break
 
-            output = self._run_test(test)
+            output = runner(test)[1]
 
             options[-1] = _("test again")
 
